import * as sqlops from 'sqlops';
import { AzureAccount, AzureAccountProviderMetadata, AzureAccountSecurityTokenCollection } from './interfaces';
import TokenCache from './tokenCache';
export declare class AzureAccountProvider implements sqlops.AccountProvider {
    private _metadata;
    private _tokenCache;
    private static WorkSchoolAccountType;
    private static MicrosoftAccountType;
    private static AadCommonTenant;
    private _args;
    private _autoOAuthCancelled;
    private _commonAuthorityUrl;
    private _inProgressAutoOAuth;
    private _isInitialized;
    constructor(_metadata: AzureAccountProviderMetadata, _tokenCache: TokenCache);
    autoOAuthCancelled(): Thenable<void>;
    /**
     * Clears all tokens that belong to the given account from the token cache
     * @param {"data".AccountKey} accountKey Key identifying the account to delete tokens for
     * @returns {Thenable<void>} Promise to clear requested tokens from the token cache
     */
    clear(accountKey: sqlops.AccountKey): Thenable<void>;
    /**
     * Clears the entire token cache. Invoked by command palette action.
     * @returns {Thenable<void>} Promise to clear the token cache
     */
    clearTokenCache(): Thenable<void>;
    getSecurityToken(account: AzureAccount): Thenable<AzureAccountSecurityTokenCollection>;
    initialize(restoredAccounts: sqlops.Account[]): Thenable<sqlops.Account[]>;
    prompt(): Thenable<AzureAccount>;
    refresh(account: AzureAccount): Thenable<AzureAccount>;
    private cancelAutoOAuth();
    private clearAccountTokens(accountKey);
    private doIfInitialized<T>(op);
    private getAccessTokens(account);
    private getDeviceLoginUserCode();
    private getDeviceLoginToken(oAuth, isAddAccount);
    private getTenants(userId, homeTenant);
    /**
     * Retrieves a token for the given user ID for the specific tenant ID. If the token can, it
     * will be retrieved from the cache as per the ADAL API. AFAIK, the ADAL API will also utilize
     * the refresh token if there aren't any unexpired tokens to use.
     * @param {string} userId ID of the user to get a token for
     * @param {string} tenantId Tenant to get the token for
     * @param {string} resourceId ID of the resource the token will be good for
     * @returns {Thenable<TokenResponse>} Promise to return a token. Rejected if retrieving the token fails.
     */
    private getToken(userId, tenantId, resourceId);
    /**
     * Performs a web request using the provided bearer token
     * @param {TokenResponse} accessToken Bearer token for accessing the provided URI
     * @param {string} uri URI to access
     * @returns {Thenable<any>} Promise to return the deserialized body of the request. Rejected if error occurred.
     */
    private makeWebRequest(accessToken, uri);
    private signIn(isAddAccount);
}
