import * as adal from 'adal-node';
import * as sqlops from 'sqlops';
export default class TokenCache implements adal.TokenCache {
    private _credentialProvider;
    private _credentialServiceKey;
    private _cacheSerializationPath;
    private static CipherAlgorithm;
    private static CipherAlgorithmIvLength;
    private static CipherKeyLength;
    private static FsOptions;
    private _activeOperation;
    constructor(_credentialProvider: sqlops.CredentialProvider, _credentialServiceKey: string, _cacheSerializationPath: string);
    add(entries: adal.TokenResponse[], callback: (error: Error, result: boolean) => void): void;
    clear(): Thenable<void>;
    find(query: any, callback: (error: Error, results: any[]) => void): void;
    /**
     * Wrapper to make callback-based find method into a thenable method
     * @param query Partial object to use to look up tokens. Ideally should be partial of adal.TokenResponse
     * @returns {Thenable<any[]>} Promise to return the matching adal.TokenResponse objects.
     *     Rejected if an error was sent in the callback
     */
    findThenable(query: any): Thenable<any[]>;
    remove(entries: adal.TokenResponse[], callback: (error: Error, result: null) => void): void;
    /**
     * Wrapper to make callback-based remove method into a thenable method
     * @param {TokenResponse[]} entries Array of entries to remove from the token cache
     * @returns {Thenable<void>} Promise to remove the given tokens from the token cache
     *     Rejected if an error was sent in the callback
     */
    removeThenable(entries: adal.TokenResponse[]): Thenable<void>;
    private static findByKeyHelper(entry1, entry2);
    private static findByPartial(entry, query);
    private doOperation<T>(op);
    private addToCache(cache, entries);
    private getOrCreateEncryptionParams();
    private readCache();
    private removeFromCache(cache, entries);
    private writeCache(cache);
}
