/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the Source EULA. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const os = require("os");
const constants = require("./constants");
const azureAccountProviderService_1 = require("./account-provider/azureAccountProviderService");
// The function is a duplicate of \src\paths.js. IT would be better to import path.js but it doesn't
// work for now because the extension is running in different process.
function getAppDataPath() {
    var platform = process.platform;
    switch (platform) {
        case 'win32': return process.env['APPDATA'] || path.join(process.env['USERPROFILE'], 'AppData', 'Roaming');
        case 'darwin': return path.join(os.homedir(), 'Library', 'Application Support');
        case 'linux': return process.env['XDG_CONFIG_HOME'] || path.join(os.homedir(), '.config');
        default: throw new Error('Platform not supported');
    }
}
exports.getAppDataPath = getAppDataPath;
function getDefaultLogLocation() {
    return path.join(getAppDataPath(), 'sqlops');
}
exports.getDefaultLogLocation = getDefaultLogLocation;
// EXTENSION ACTIVATION ////////////////////////////////////////////////////
function activate(context) {
    // Create the folder for storing the token caches
    let storagePath = path.join(getDefaultLogLocation(), constants.extensionName);
    try {
        if (!fs.existsSync(storagePath)) {
            fs.mkdirSync(storagePath);
            console.log('Initialized Azure account extension storage.');
        }
    }
    catch (e) {
        console.error(`Initialization of Azure account extension storage failed: ${e}`);
        console.error('Azure accounts will not be available');
        return;
    }
    // Create the provider service and activate
    const accountProviderService = new azureAccountProviderService_1.AzureAccountProviderService(context, storagePath);
    context.subscriptions.push(accountProviderService);
    accountProviderService.activate();
}
exports.activate = activate;
//# sourceMappingURL=https://ticino.blob.core.windows.net/sourcemaps/7099922c35bc3b3e223228c972cc03cee5abbe25/extensions/account-provider-azure/out/main.js.map
