/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the Source EULA. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const dataprotocol_client_1 = require("dataprotocol-client");
const UUID = require("vscode-languageclient/lib/utils/uuid");
const sqlops = require("sqlops");
const telemetry_1 = require("./telemetry");
const Utils = require("./utils");
const contracts_1 = require("./contracts");
class TelemetryFeature {
    constructor(_client) {
        this._client = _client;
    }
    fillClientCapabilities(capabilities) {
        Utils.ensure(capabilities, 'telemetry').telemetry = true;
    }
    initialize() {
        this._client.onNotification(contracts_1.TelemetryNotification.type, e => {
            telemetry_1.Telemetry.sendTelemetryEvent(e.params.eventName, e.params.properties, e.params.measures);
        });
    }
}
exports.TelemetryFeature = TelemetryFeature;
class AgentServicesFeature extends dataprotocol_client_1.SqlOpsFeature {
    constructor(client) {
        super(client, AgentServicesFeature.messagesTypes);
    }
    fillClientCapabilities(capabilities) {
        // this isn't explicitly necessary
        // ensure(ensure(capabilities, 'connection')!, 'agentServices')!.dynamicRegistration = true;
    }
    initialize(capabilities) {
        this.register(this.messages, {
            id: UUID.generateUuid(),
            registerOptions: undefined
        });
    }
    registerProvider(options) {
        const client = this._client;
        let getJobs = (ownerUri) => {
            let params = { ownerUri: ownerUri, jobId: null };
            return client.sendRequest(contracts_1.AgentJobsRequest.type, params).then(r => r, e => {
                client.logFailedRequest(contracts_1.AgentJobsRequest.type, e);
                return Promise.resolve(undefined);
            });
        };
        let getJobHistory = (connectionUri, jobID) => {
            let params = { ownerUri: connectionUri, jobId: jobID };
            return client.sendRequest(contracts_1.AgentJobHistoryRequest.type, params).then(r => r, e => {
                client.logFailedRequest(contracts_1.AgentJobHistoryRequest.type, e);
                return Promise.resolve(undefined);
            });
        };
        let jobAction = (connectionUri, jobName, action) => {
            let params = { ownerUri: connectionUri, jobName: jobName, action: action };
            return client.sendRequest(contracts_1.AgentJobActionRequest.type, params).then(r => r, e => {
                client.logFailedRequest(contracts_1.AgentJobActionRequest.type, e);
                return Promise.resolve(undefined);
            });
        };
        return sqlops.dataprotocol.registerAgentServicesProvider({
            providerId: client.providerId,
            getJobs,
            getJobHistory,
            jobAction
        });
    }
}
AgentServicesFeature.messagesTypes = [
    contracts_1.AgentJobsRequest.type,
    contracts_1.AgentJobHistoryRequest.type,
    contracts_1.AgentJobActionRequest.type
];
exports.AgentServicesFeature = AgentServicesFeature;
//# sourceMappingURL=https://ticino.blob.core.windows.net/sourcemaps/7099922c35bc3b3e223228c972cc03cee5abbe25/extensions/mssql/out/features.js.map
