import { PlatformInformation } from 'service-downloader/out/platform';
import { ErrorAction, ErrorHandler, Message, CloseAction } from 'vscode-languageclient';
export interface ITelemetryEventProperties {
    [key: string]: string;
}
export interface ITelemetryEventMeasures {
    [key: string]: number;
}
/**
 * Filters error paths to only include source files. Exported to support testing
 */
export declare function FilterErrorPath(line: string): string;
export declare class Telemetry {
    private static reporter;
    private static userId;
    private static platformInformation;
    private static disabled;
    static getUserId(): Promise<string>;
    static getPlatformInformation(): Promise<PlatformInformation>;
    /**
     * Disable telemetry reporting
     */
    static disable(): void;
    /**
     * Initialize the telemetry reporter for use.
     */
    static initialize(): void;
    /**
     * Send a telemetry event for an exception
     */
    static sendTelemetryEventForException(err: any, methodName: string, extensionConfigName: string): void;
    /**
     * Send a telemetry event using application insights
     */
    static sendTelemetryEvent(eventName: string, properties?: ITelemetryEventProperties, measures?: ITelemetryEventMeasures): void;
}
/**
 * Handle Language Service client errors
 * @class LanguageClientErrorHandler
 */
export declare class LanguageClientErrorHandler implements ErrorHandler {
    /**
     * Show an error message prompt with a link to known issues wiki page
     * @memberOf LanguageClientErrorHandler
     */
    showOnErrorPrompt(): void;
    /**
     * Callback for language service client error
     *
     * @param {Error} error
     * @param {Message} message
     * @param {number} count
     * @returns {ErrorAction}
     *
     * @memberOf LanguageClientErrorHandler
     */
    error(error: Error, message: Message, count: number): ErrorAction;
    /**
     * Callback for language service client closed
     *
     * @returns {CloseAction}
     *
     * @memberOf LanguageClientErrorHandler
     */
    closed(): CloseAction;
}
